@@ -30,6 +30,7 @@
 
 #include "config.h"
 #include "gromit-mpx.h"
+#include "build-config.h"
 
 #define KEY_DFLT_SHOW_INTRO_ON_STARTUP TRUE
 
@@ -124,25 +125,31 @@ void parse_config (GromitData *data)
   GdkRGBA *fg_color=NULL;
   guint width, arrowsize, minwidth;
 
+  /* try user config location */
   filename = g_strjoin (G_DIR_SEPARATOR_S,
                         g_get_user_config_dir(), "gromit-mpx.cfg", NULL);
-  file = open (filename, O_RDONLY);
+  if ((file = open(filename, O_RDONLY)) < 0)
+      g_print("Could not open user config %s: %s\n", filename, g_strerror (errno));
+  else
+      g_print("Using user config %s\n", filename);
 
-  if (file < 0)
-    {
-      g_printerr ("Could not open %s: %s\n", filename, g_strerror (errno));
-      /* try global config file */
-      g_free (filename);
-      filename = g_strdup ("/etc/gromit-mpx/gromit-mpx.cfg");
-      file = open (filename, O_RDONLY);
 
-      if (file < 0)
-        {
-          g_printerr ("Could not open %s: %s\n", filename, g_strerror (errno));
-          g_free (filename);
-          return;
-        }
-    }
+  /* try global config file */
+  if (file < 0) {
+      g_free(filename);
+      filename = g_strdup (SYSCONFDIR "/gromit-mpx/gromit-mpx.cfg");
+      if ((file = open(filename, O_RDONLY)) < 0)
+	  g_print("Could not open system config %s: %s\n", filename, g_strerror (errno));
+      else
+	  g_print("Using system config %s\n", filename);
+  }
+
+  /* was the last possibility, no use to go on */
+  if (file < 0) {
+      g_printerr("No usable config found, leaving tools unconfigured!\n");
+      g_free(filename);
+      return;
+  }
 
   scanner = g_scanner_new (NULL);
   scanner->input_name = filename;
@@ -441,6 +448,10 @@ void read_keyfile(GromitData *data)
     }
 
     data->show_intro_on_startup = g_key_file_get_boolean (key_file, "General", "ShowIntroOnStartup", &error);
+    data->opacity = g_key_file_get_double (key_file, "Drawing", "Opacity", &error);
+    // 0.0 on not-found, but anyway, also don't use 0.0 when user-set
+    if(data->opacity == 0)
+	data->opacity = DEFAULT_OPACITY;
 
  cleanup:
     g_free(filename);
@@ -457,6 +468,7 @@ void write_keyfile(GromitData *data)
     GKeyFile *key_file = g_key_file_new ();
 
     g_key_file_set_boolean (key_file, "General", "ShowIntroOnStartup", data->show_intro_on_startup);
+    g_key_file_set_double (key_file, "Drawing", "Opacity", data->opacity);
 
     // Save as a file.
     if (!g_key_file_save_to_file (key_file, filename, &error)) {
